/*
 * Python Wrapper file for Core Library
 *
 * Author: Zilin Du
 *
 * $Id: corelib.i,v 1.2 2005/05/09 21:15:13 exact Exp $
 */

/* File : example.i */
%module corelib 

%{
#include <CORE/Expr.h>
using namespace CORE;
%}

%include "std_string.i"
/* class extLong */
class extLong {
public:
	extLong();
  extLong(long);
  
  long toLong();
  long asLong();
  bool isInfty();
  bool isTiny();
  bool isNaN();
};

%extend extLong {
	std::string __str__() { return self->toString(); }
	/* operators */
	extLong __add__(extLong* rhs) { return (*self) + (*rhs); }
	extLong __sub__(extLong* rhs) { return (*self) - (*rhs); }
	extLong __mul__(extLong* rhs) { return (*self) * (*rhs); }
	extLong __div__(extLong* rhs) { return (*self) / (*rhs); }
	extLong __neg__() { return -(*self); }
  int __cmp__(extLong* rhs) { return self->cmp(*rhs); }
  int __cmp__(long rhs) { return self->cmp(rhs); }
};

/* class BigInt */
class BigInt {
public:
	BigInt();
	BigInt(long);
	BigInt(double);
	BigInt(const std::string&);

	long longValue();
	double doubleValue();
};

%extend BigInt {
	std::string __str__() { return self->get_str(); }
	double __float__() { return self->doubleValue(); }
	int fromString(const std::string& s) { return self->set_str(s.c_str()); }
	std::string toString() { return self->get_str(); }
	/* operators */
	BigInt __add__(BigInt* rhs) { return (*self) + (*rhs); }
	BigInt __sub__(BigInt* rhs) { return (*self) - (*rhs); }
	BigInt __mul__(BigInt* rhs) { return (*self) * (*rhs); }
	BigInt __div__(BigInt* rhs) { return (*self) / (*rhs); }
	BigInt __mod__(BigInt* rhs) { return (*self) % (*rhs); }
	BigInt __and__(BigInt* rhs) { return (*self) & (*rhs); }
	BigInt __or__(BigInt* rhs) { return (*self) | (*rhs); }
	BigInt __xor__(BigInt* rhs) { return (*self) ^ (*rhs); }
	BigInt __pow__(unsigned long rhs) { return pow((*self), rhs); }
	BigInt __lshift__(unsigned long rhs) { return (*self) << rhs; }
	BigInt __rshift__(unsigned long rhs) { return (*self) >> rhs; }
	BigInt __neg__() { return -(*self); }
	BigInt __pos__() { return +(*self); }
	BigInt __abs__() { return abs(*self); }
  int __cmp__(BigInt* rhs) { return cmp(*self, *rhs); }
	/* extra operators (long) */
	BigInt __add__(long rhs) { return (*self) + rhs; }
	BigInt __radd__(long rhs) { return rhs + (*self) ; }
	BigInt __sub__(long rhs) { return (*self) - rhs; }
	BigInt __rsub__(long rhs) { return rhs - (*self) ; }
	BigInt __mul__(long rhs) { return (*self) * rhs; }
	BigInt __rmul__(long rhs) { return rhs * (*self) ; }
	BigInt __div__(long rhs) { return (*self) / rhs; }
	BigInt __rdiv__(long rhs) { return rhs / (*self) ; }
	BigInt __mod__(long rhs) { return (*self) % rhs; }
	BigInt __rmod__(long rhs) { return rhs % (*self) ; }
	BigInt __and__(long rhs) { return (*self) & rhs; }
	BigInt __rand__(long rhs) { return rhs & (*self) ; }
	BigInt __or__(long rhs) { return (*self) | rhs; }
	BigInt __ror__(long rhs) { return rhs | (*self) ; }
	BigInt __xor__(long rhs) { return (*self) ^ rhs; }
	BigInt __rxor__(long rhs) { return rhs ^ (*self) ; }
  int __cmp__(long rhs) { return cmp(*self, rhs); }
	/* extra operators (double) */
	BigInt __add__(double rhs) { return (*self) + rhs; }
	BigInt __radd__(double rhs) { return rhs + (*self) ; }
	BigInt __sub__(double rhs) { return (*self) - rhs; }
	BigInt __rsub__(double rhs) { return rhs - (*self) ; }
	BigInt __mul__(double rhs) { return (*self) * rhs; }
	BigInt __rmul__(double rhs) { return rhs * (*self) ; }
	BigInt __div__(double rhs) { return (*self) / rhs; }
	BigInt __rdiv__(double rhs) { return rhs / (*self) ; }
	BigInt __mod__(double rhs) { return (*self) % rhs; }
	BigInt __rmod__(double rhs) { return rhs % (*self) ; }
	BigInt __and__(double rhs) { return (*self) & rhs; }
	BigInt __rand__(double rhs) { return rhs & (*self) ; }
	BigInt __or__(double rhs) { return (*self) | rhs; }
	BigInt __ror__(double rhs) { return rhs | (*self) ; }
	BigInt __xor__(double rhs) { return (*self) ^ rhs; }
	BigInt __rxor__(double rhs) { return rhs ^ (*self) ; }
  int __cmp__(double rhs) { return cmp(*self, rhs); }
	/* extra operators (std::string) */
	BigInt __add__(const std::string& rhs) { return (*self) + rhs; }
	BigInt __radd__(const std::string& rhs) { return rhs + (*self) ; }
	BigInt __sub__(const std::string& rhs) { return (*self) - rhs; }
	BigInt __rsub__(const std::string& rhs) { return rhs - (*self) ; }
	BigInt __mul__(const std::string& rhs) { return (*self) * rhs; }
	BigInt __rmul__(const std::string& rhs) { return rhs * (*self) ; }
	BigInt __div__(const std::string& rhs) { return (*self) / rhs; }
	BigInt __rdiv__(const std::string& rhs) { return rhs / (*self) ; }
	BigInt __mod__(const std::string& rhs) { return (*self) % rhs; }
	BigInt __rmod__(const std::string& rhs) { return rhs % (*self) ; }
	BigInt __and__(const std::string& rhs) { return (*self) & rhs; }
	BigInt __rand__(const std::string& rhs) { return rhs & (*self) ; }
	BigInt __or__(const std::string& rhs) { return (*self) | rhs; }
	BigInt __ror__(const std::string& rhs) { return rhs | (*self) ; }
	BigInt __xor__(const std::string& rhs) { return (*self) ^ rhs; }
	BigInt __rxor__(const std::string& rhs) { return rhs ^ (*self) ; }
  int __cmp__(const std::string& rhs) { return cmp(*self, rhs); }
}; 

int sign(const BigInt&);
int cmp(const BigInt&, const BigInt&);
int cmp(const BigInt&, long);
int cmp(const BigInt&, double);
int cmp(const BigInt&, const std::string&);
int cmp(long, const BigInt&);
int cmp(double, const BigInt&);
int cmp(const std::string&, const BigInt&);

/* class BigRat */
class BigRat {
public:
	BigRat();
	BigRat(long);
	BigRat(double);
	BigRat(const std::string&);
	BigRat(const BigInt&);

	BigRat(long, long);
	BigRat(long, double);
	BigRat(long, const std::string&);
	BigRat(long, const BigInt&);
	BigRat(double, long);
	BigRat(double, double);
	BigRat(double, const std::string&);
	BigRat(double, const BigInt&);
	BigRat(const std::string&, long);
	BigRat(const std::string&, double);
	BigRat(const std::string&, const std::string&);
	BigRat(const std::string&, const BigInt&);
	BigRat(const BigInt&, long);
	BigRat(const BigInt&, double);
	BigRat(const BigInt&, const std::string&);
	BigRat(const BigInt&, const BigInt&);

	long longValue();
	double doubleValue();
	BigInt BigIntValue();

  void canonicalize();
};

%extend BigRat {
	std::string __str__() { return self->get_str(); }
	double __float__() { return self->doubleValue(); }
	int fromString(const std::string& s) { return self->set_str(s.c_str()); }
	std::string toString() { return self->get_str(); }
	/* operators */
	BigRat __add__(BigRat* rhs) { return (*self) + (*rhs); }
	BigRat __sub__(BigRat* rhs) { return (*self) - (*rhs); }
	BigRat __mul__(BigRat* rhs) { return (*self) * (*rhs); }
	BigRat __div__(BigRat* rhs) { return (*self) / (*rhs); }
	BigRat __lshift__(unsigned long rhs) { return (*self) << rhs; }
	BigRat __rshift__(unsigned long rhs) { return (*self) >> rhs; }
	BigRat __neg__() { return -(*self); }
	BigRat __pos__() { return +(*self); }
	BigRat __abs__() { return abs(*self); }
  int __cmp__(BigRat* rhs) { return cmp(*self, *rhs); }
	/* extra operators (long) */
	BigRat __add__(long rhs) { return (*self) + rhs; }
	BigRat __radd__(long rhs) { return rhs + (*self) ; }
	BigRat __sub__(long rhs) { return (*self) - rhs; }
	BigRat __rsub__(long rhs) { return rhs - (*self) ; }
	BigRat __mul__(long rhs) { return (*self) * rhs; }
	BigRat __rmul__(long rhs) { return rhs * (*self) ; }
	BigRat __div__(long rhs) { return (*self) / rhs; }
	BigRat __rdiv__(long rhs) { return rhs / (*self) ; }
  int __cmp__(long rhs) { return cmp(*self, rhs); }
	/* extra operators (double) */
	BigRat __add__(double rhs) { return (*self) + rhs; }
	BigRat __radd__(double rhs) { return rhs + (*self) ; }
	BigRat __sub__(double rhs) { return (*self) - rhs; }
	BigRat __rsub__(double rhs) { return rhs - (*self) ; }
	BigRat __mul__(double rhs) { return (*self) * rhs; }
	BigRat __rmul__(double rhs) { return rhs * (*self) ; }
	BigRat __div__(double rhs) { return (*self) / rhs; }
	BigRat __rdiv__(double rhs) { return rhs / (*self) ; }
  int __cmp__(double rhs) { return cmp(*self, rhs); }
	/* extra operators (std::string) */
	BigRat __add__(const std::string& rhs) { return (*self) + rhs; }
	BigRat __radd__(const std::string& rhs) { return rhs + (*self) ; }
	BigRat __sub__(const std::string& rhs) { return (*self) - rhs; }
	BigRat __rsub__(const std::string& rhs) { return rhs - (*self) ; }
	BigRat __mul__(const std::string& rhs) { return (*self) * rhs; }
	BigRat __rmul__(const std::string& rhs) { return rhs * (*self) ; }
	BigRat __div__(const std::string& rhs) { return (*self) / rhs; }
	BigRat __rdiv__(const std::string& rhs) { return rhs / (*self) ; }
  int __cmp__(const std::string& rhs) { return cmp(*self, rhs); }
	/* extra operators (BigInt) */
	BigRat __add__(const BigInt& rhs) { return (*self) + rhs; }
	BigRat __radd__(const BigInt& rhs) { return rhs + (*self) ; }
	BigRat __sub__(const BigInt& rhs) { return (*self) - rhs; }
	BigRat __rsub__(const BigInt& rhs) { return rhs - (*self) ; }
	BigRat __mul__(const BigInt& rhs) { return (*self) * rhs; }
	BigRat __rmul__(const BigInt& rhs) { return rhs * (*self) ; }
	BigRat __div__(const BigInt& rhs) { return (*self) / rhs; }
	BigRat __rdiv__(const BigInt& rhs) { return rhs / (*self) ; }
  int __cmp__(const BigInt& rhs) { return cmp(*self, rhs); }
}; 

int sign(const BigRat&);
int cmp(const BigRat&, const BigRat&);
int cmp(const BigRat&, long);
int cmp(const BigRat&, double);
int cmp(const BigRat&, const std::string&);
int cmp(const BigRat&, const BigInt&);
int cmp(long, const BigRat&);
int cmp(double, const BigRat&);
int cmp(const std::string&, const BigRat&);
int cmp(const BigInt&, const BigRat&);
BigInt numerator(const BigRat&);
BigInt denominator(const BigRat&);

/* class BigFloat */
class BigFloat {
public:
	BigFloat();
	BigFloat(long);
	BigFloat(double);
	BigFloat(const std::string&);
	BigFloat(const BigInt&, unsigned long u=0, long l=0);
	BigFloat(const BigRat&, const extLong& r=defRelPrec, const extLong& a=defAbsPrec);

	void fromString(const char* s, const extLong& p=defBigFloatInputDigits);
  std::string toString(long prec=defBigFloatOutputDigits, bool sci=false);
	long longValue();
	double doubleValue();
	BigInt BigIntValue();
	BigRat BigRatValue();

  bool isZeroIn();
	const BigInt& m();
  unsigned long err();
  long exp();
  bool isExact();
  BigFloat& makeExact();
  BigFloat& makeInexact();
  extLong lMSB();
  extLong uMSB();
  extLong MSB();
  extLong flrLgErr();
  extLong clLgErr();
  
  BigFloat div(const BigFloat& x, const extLong& r);
  BigFloat div2();
  BigFloat sqrt(const extLong& a);
  BigFloat sqrt(const extLong& a, const BigFloat& init);
  
  void approx(const BigInt& I, const extLong& r, const extLong& a);
  void approx(const BigRat& I, const extLong& r, const extLong& a);
  void approx(const BigFloat& I, const extLong& r, const extLong& a);
  
  static BigFloat exp2(int e);
};

%extend BigFloat {
	std::string __str__() { return self->toString(); }
	double __float__() { return self->doubleValue(); }
	
	/* operators */
	BigFloat __add__(BigFloat* rhs) { return (*self) + (*rhs); }
	BigFloat __sub__(BigFloat* rhs) { return (*self) - (*rhs); }
	BigFloat __mul__(BigFloat* rhs) { return (*self) * (*rhs); }
	BigFloat __div__(BigFloat* rhs) { return (*self) / (*rhs); }
	BigFloat __pow__(unsigned long rhs) { return pow((*self), rhs); }
	BigFloat __neg__() { return -(*self); }
	BigFloat __pos__() { return +(*self); }
	BigFloat __abs__() { return abs(*self); }
  int __cmp__(BigFloat* rhs) { return cmp(*self, *rhs); }
	/* extra operators (long) */
	BigFloat __add__(long rhs) { return (*self) + rhs; }
	BigFloat __radd__(long rhs) { return rhs + (*self) ; }
	BigFloat __sub__(long rhs) { return (*self) - rhs; }
	BigFloat __rsub__(long rhs) { return rhs - (*self) ; }
	BigFloat __mul__(long rhs) { return (*self) * rhs; }
	BigFloat __rmul__(long rhs) { return rhs * (*self) ; }
	BigFloat __div__(long rhs) { return (*self) / rhs; }
	BigFloat __rdiv__(long rhs) { return rhs / (*self) ; }
  int __cmp__(long rhs) { return cmp(*self, rhs); }
	/* extra operators (double) */
	BigFloat __add__(double rhs) { return (*self) + rhs; }
	BigFloat __radd__(double rhs) { return rhs + (*self) ; }
	BigFloat __sub__(double rhs) { return (*self) - rhs; }
	BigFloat __rsub__(double rhs) { return rhs - (*self) ; }
	BigFloat __mul__(double rhs) { return (*self) * rhs; }
	BigFloat __rmul__(double rhs) { return rhs * (*self) ; }
	BigFloat __div__(double rhs) { return (*self) / rhs; }
	BigFloat __rdiv__(double rhs) { return rhs / (*self) ; }
  int __cmp__(double rhs) { return cmp(*self, rhs); }
	/* extra operators (std::string) */
	BigFloat __add__(const std::string& rhs) { return (*self) + rhs; }
	BigFloat __radd__(const std::string& rhs) { return rhs + (*self) ; }
	BigFloat __sub__(const std::string& rhs) { return (*self) - rhs; }
	BigFloat __rsub__(const std::string& rhs) { return rhs - (*self) ; }
	BigFloat __mul__(const std::string& rhs) { return (*self) * rhs; }
	BigFloat __rmul__(const std::string& rhs) { return rhs * (*self) ; }
	BigFloat __div__(const std::string& rhs) { return (*self) / rhs; }
	BigFloat __rdiv__(const std::string& rhs) { return rhs / (*self) ; }
  int __cmp__(const std::string& rhs) { return cmp(*self, rhs); }
	/* extra operators (BigInt) */
	BigFloat __add__(const BigInt& rhs) { return (*self) + rhs; }
	BigFloat __radd__(const BigInt& rhs) { return rhs + (*self) ; }
	BigFloat __sub__(const BigInt& rhs) { return (*self) - rhs; }
	BigFloat __rsub__(const BigInt& rhs) { return rhs - (*self) ; }
	BigFloat __mul__(const BigInt& rhs) { return (*self) * rhs; }
	BigFloat __rmul__(const BigInt& rhs) { return rhs * (*self) ; }
	BigFloat __div__(const BigInt& rhs) { return (*self) / rhs; }
	BigFloat __rdiv__(const BigInt& rhs) { return rhs / (*self) ; }
  int __cmp__(const BigInt& rhs) { return cmp(*self, rhs); }
	/* extra operators (BigRat) */
	BigFloat __add__(const BigRat& rhs) { return (*self) + rhs; }
	BigFloat __radd__(const BigRat& rhs) { return rhs + (*self) ; }
	BigFloat __sub__(const BigRat& rhs) { return (*self) - rhs; }
	BigFloat __rsub__(const BigRat& rhs) { return rhs - (*self) ; }
	BigFloat __mul__(const BigRat& rhs) { return (*self) * rhs; }
	BigFloat __rmul__(const BigRat& rhs) { return rhs * (*self) ; }
	BigFloat __div__(const BigRat& rhs) { return (*self) / rhs; }
	BigFloat __rdiv__(const BigRat& rhs) { return rhs / (*self) ; }
  int __cmp__(const BigRat& rhs) { return cmp(*self, rhs); }
}; 

BigFloat sqrt(BigFloat);
int sign(const BigFloat&);
int cmp(const BigFloat&, const BigFloat&);
int cmp(const BigFloat&, long);
int cmp(const BigFloat&, double);
int cmp(const BigFloat&, const std::string&);
int cmp(const BigFloat&, const BigInt&);
int cmp(const BigFloat&, const BigRat&);
int cmp(long, const BigFloat&);
int cmp(double, const BigFloat&);
int cmp(const std::string&, const BigFloat&);
int cmp(const BigInt&, const BigFloat&);
int cmp(const BigRat&, const BigFloat&);

/* class Real */
class Real {
public:
	Real();
	Real(long);
	Real(double);
	Real(const std::string&);
	Real(const BigInt&);
	Real(const BigRat&);
	Real(const BigFloat&);

	void fromString(const char* s, const extLong& p=defInputDigits);
  std::string toString(long prec=defOutputDigits, bool sci=false);
	long longValue();
	double doubleValue();
	BigInt BigIntValue();
	BigRat BigRatValue();
	BigFloat BigFloatValue();

  bool isZeroIn();
	BigInt getMantissa();
  long getExponent();
  bool isExact();
  extLong lMSB();
  extLong uMSB();
  extLong MSB();
  extLong flrLgErr();
  extLong clLgErr();
  
  Real div(const Real& x, const extLong& r);
  Real sqrt(const extLong& a);
  Real sqrt(const extLong& a, const BigFloat& init);
  
  Real approx(const extLong& r=defRelPrec, const extLong& a=defAbsPrec);

  unsigned long degree();
  unsigned long length();
  unsigned long height();
};

%extend Real {
	std::string __str__() { return self->toString(); }
	double __float__() { return self->doubleValue(); }
	/* operators */
	Real __add__(Real* rhs) { return (*self) + (*rhs); }
	Real __sub__(Real* rhs) { return (*self) - (*rhs); }
	Real __mul__(Real* rhs) { return (*self) * (*rhs); }
	Real __div__(Real* rhs) { return (*self) / (*rhs); }
	Real __pow__(unsigned long rhs) { return pow((*self), rhs); }
	Real __neg__() { return -(*self); }
	Real __pos__() { return +(*self); }
	Real __abs__() { return abs(*self); }
  int __cmp__(Real* rhs) { return cmp(*self, *rhs); }
	/* extra operators (long) */
	Real __add__(long rhs) { return (*self) + rhs; }
	Real __radd__(long rhs) { return rhs + (*self) ; }
	Real __sub__(long rhs) { return (*self) - rhs; }
	Real __rsub__(long rhs) { return rhs - (*self) ; }
	Real __mul__(long rhs) { return (*self) * rhs; }
	Real __rmul__(long rhs) { return rhs * (*self) ; }
	Real __div__(long rhs) { return (*self) / rhs; }
	Real __rdiv__(long rhs) { return rhs / (*self) ; }
  int __cmp__(long rhs) { return cmp(*self, rhs); }
	/* extra operators (double) */
	Real __add__(double rhs) { return (*self) + rhs; }
	Real __radd__(double rhs) { return rhs + (*self) ; }
	Real __sub__(double rhs) { return (*self) - rhs; }
	Real __rsub__(double rhs) { return rhs - (*self) ; }
	Real __mul__(double rhs) { return (*self) * rhs; }
	Real __rmul__(double rhs) { return rhs * (*self) ; }
	Real __div__(double rhs) { return (*self) / rhs; }
	Real __rdiv__(double rhs) { return rhs / (*self) ; }
  int __cmp__(double rhs) { return cmp(*self, rhs); }
	/* extra operators (std::string) */
	Real __add__(const std::string& rhs) { return (*self) + rhs; }
	Real __radd__(const std::string& rhs) { return rhs + (*self) ; }
	Real __sub__(const std::string& rhs) { return (*self) - rhs; }
	Real __rsub__(const std::string& rhs) { return rhs - (*self) ; }
	Real __mul__(const std::string& rhs) { return (*self) * rhs; }
	Real __rmul__(const std::string& rhs) { return rhs * (*self) ; }
	Real __div__(const std::string& rhs) { return (*self) / rhs; }
	Real __rdiv__(const std::string& rhs) { return rhs / (*self) ; }
  int __cmp__(const std::string& rhs) { return cmp(*self, rhs); }
	/* extra operators (BigInt) */
	Real __add__(const BigInt& rhs) { return (*self) + rhs; }
	Real __radd__(const BigInt& rhs) { return rhs + (*self) ; }
	Real __sub__(const BigInt& rhs) { return (*self) - rhs; }
	Real __rsub__(const BigInt& rhs) { return rhs - (*self) ; }
	Real __mul__(const BigInt& rhs) { return (*self) * rhs; }
	Real __rmul__(const BigInt& rhs) { return rhs * (*self) ; }
	Real __div__(const BigInt& rhs) { return (*self) / rhs; }
	Real __rdiv__(const BigInt& rhs) { return rhs / (*self) ; }
  int __cmp__(const BigInt& rhs) { return cmp(*self, rhs); }
	/* extra operators (BigRat) */
	Real __add__(const BigRat& rhs) { return (*self) + rhs; }
	Real __radd__(const BigRat& rhs) { return rhs + (*self) ; }
	Real __sub__(const BigRat& rhs) { return (*self) - rhs; }
	Real __rsub__(const BigRat& rhs) { return rhs - (*self) ; }
	Real __mul__(const BigRat& rhs) { return (*self) * rhs; }
	Real __rmul__(const BigRat& rhs) { return rhs * (*self) ; }
	Real __div__(const BigRat& rhs) { return (*self) / rhs; }
	Real __rdiv__(const BigRat& rhs) { return rhs / (*self) ; }
  int __cmp__(const BigRat& rhs) { return cmp(*self, rhs); }
	/* extra operators (BigFloat) */
	Real __add__(const BigFloat& rhs) { return (*self) + rhs; }
	Real __radd__(const BigFloat& rhs) { return rhs + (*self) ; }
	Real __sub__(const BigFloat& rhs) { return (*self) - rhs; }
	Real __rsub__(const BigFloat& rhs) { return rhs - (*self) ; }
	Real __mul__(const BigFloat& rhs) { return (*self) * rhs; }
	Real __rmul__(const BigFloat& rhs) { return rhs * (*self) ; }
	Real __div__(const BigFloat& rhs) { return (*self) / rhs; }
	Real __rdiv__(const BigFloat& rhs) { return rhs / (*self) ; }
  int __cmp__(const BigFloat& rhs) { return cmp(*self, rhs); }
}; 

Real sqrt(Real);
bool isZero(Real);
Real fabs(Real);
BigInt floor(Real);
BigInt ceil(Real);
int sign(const Real&);
int cmp(const Real&, const Real&);
int cmp(const Real&, long);
int cmp(const Real&, double);
int cmp(const Real&, const std::string&);
int cmp(const Real&, const BigInt&);
int cmp(const Real&, const BigRat&);
int cmp(const Real&, const BigFloat&);
int cmp(long, const Real&);
int cmp(double, const Real&);
int cmp(const std::string&, const Real&);
int cmp(const BigInt&, const Real&);
int cmp(const BigRat&, const Real&);
int cmp(const BigFloat&, const Real&);

/* class Expr */
class Expr {
public:
	Expr();
	Expr(long);
	Expr(double);
	Expr(const std::string&);
	Expr(const BigInt&);
	Expr(const BigRat&);
	Expr(const BigFloat&);

	void fromString(const char* s, const extLong& p=defInputDigits);
  std::string toString(long prec=defOutputDigits, bool sci=false);
	long longValue();
	double doubleValue();
	BigInt BigIntValue();
	BigRat BigRatValue();
	BigFloat BigFloatValue();

	BigInt getMantissa();
  long getExponent();
  
  const Real& approx(const extLong& r=defRelPrec, const extLong& a=defAbsPrec);
};

%extend Expr {
	std::string __str__() { return self->toString(); }
	double __float__() { return self->doubleValue(); }
	/* operators */
	Expr __add__(Expr* rhs) { return (*self) + (*rhs); }
	Expr __sub__(Expr* rhs) { return (*self) - (*rhs); }
	Expr __mul__(Expr* rhs) { return (*self) * (*rhs); }
	Expr __div__(Expr* rhs) { return (*self) / (*rhs); }
	Expr __pow__(unsigned long rhs) { return pow((*self), rhs); }
	Expr __neg__() { return -(*self); }
	Expr __pos__() { return +(*self); }
	Expr __abs__() { return abs(*self); }
  int __cmp__(Expr* rhs) { return cmp(*self, *rhs); }
	/* extra operators (long) */
	Expr __add__(long rhs) { return (*self) + rhs; }
	Expr __radd__(long rhs) { return rhs + (*self) ; }
	Expr __sub__(long rhs) { return (*self) - rhs; }
	Expr __rsub__(long rhs) { return rhs - (*self) ; }
	Expr __mul__(long rhs) { return (*self) * rhs; }
	Expr __rmul__(long rhs) { return rhs * (*self) ; }
	Expr __div__(long rhs) { return (*self) / rhs; }
	Expr __rdiv__(long rhs) { return rhs / (*self) ; }
  int __cmp__(long rhs) { return cmp(*self, rhs); }
	/* extra operators (double) */
	Expr __add__(double rhs) { return (*self) + rhs; }
	Expr __radd__(double rhs) { return rhs + (*self) ; }
	Expr __sub__(double rhs) { return (*self) - rhs; }
	Expr __rsub__(double rhs) { return rhs - (*self) ; }
	Expr __mul__(double rhs) { return (*self) * rhs; }
	Expr __rmul__(double rhs) { return rhs * (*self) ; }
	Expr __div__(double rhs) { return (*self) / rhs; }
	Expr __rdiv__(double rhs) { return rhs / (*self) ; }
  int __cmp__(double rhs) { return cmp(*self, rhs); }
	/* extra operators (std::string) */
	Expr __add__(const std::string& rhs) { return (*self) + rhs; }
	Expr __radd__(const std::string& rhs) { return rhs + (*self) ; }
	Expr __sub__(const std::string& rhs) { return (*self) - rhs; }
	Expr __rsub__(const std::string& rhs) { return rhs - (*self) ; }
	Expr __mul__(const std::string& rhs) { return (*self) * rhs; }
	Expr __rmul__(const std::string& rhs) { return rhs * (*self) ; }
	Expr __div__(const std::string& rhs) { return (*self) / rhs; }
	Expr __rdiv__(const std::string& rhs) { return rhs / (*self) ; }
  int __cmp__(const std::string& rhs) { return cmp(*self, rhs); }
	/* extra operators (BigInt) */
	Expr __add__(const BigInt& rhs) { return (*self) + rhs; }
	Expr __radd__(const BigInt& rhs) { return rhs + (*self) ; }
	Expr __sub__(const BigInt& rhs) { return (*self) - rhs; }
	Expr __rsub__(const BigInt& rhs) { return rhs - (*self) ; }
	Expr __mul__(const BigInt& rhs) { return (*self) * rhs; }
	Expr __rmul__(const BigInt& rhs) { return rhs * (*self) ; }
	Expr __div__(const BigInt& rhs) { return (*self) / rhs; }
	Expr __rdiv__(const BigInt& rhs) { return rhs / (*self) ; }
  int __cmp__(const BigInt& rhs) { return cmp(*self, rhs); }
	/* extra operators (BigRat) */
	Expr __add__(const BigRat& rhs) { return (*self) + rhs; }
	Expr __radd__(const BigRat& rhs) { return rhs + (*self) ; }
	Expr __sub__(const BigRat& rhs) { return (*self) - rhs; }
	Expr __rsub__(const BigRat& rhs) { return rhs - (*self) ; }
	Expr __mul__(const BigRat& rhs) { return (*self) * rhs; }
	Expr __rmul__(const BigRat& rhs) { return rhs * (*self) ; }
	Expr __div__(const BigRat& rhs) { return (*self) / rhs; }
	Expr __rdiv__(const BigRat& rhs) { return rhs / (*self) ; }
  int __cmp__(const BigRat& rhs) { return cmp(*self, rhs); }
	/* extra operators (BigFloat) */
	Expr __add__(const BigFloat& rhs) { return (*self) + rhs; }
	Expr __radd__(const BigFloat& rhs) { return rhs + (*self) ; }
	Expr __sub__(const BigFloat& rhs) { return (*self) - rhs; }
	Expr __rsub__(const BigFloat& rhs) { return rhs - (*self) ; }
	Expr __mul__(const BigFloat& rhs) { return (*self) * rhs; }
	Expr __rmul__(const BigFloat& rhs) { return rhs * (*self) ; }
	Expr __div__(const BigFloat& rhs) { return (*self) / rhs; }
	Expr __rdiv__(const BigFloat& rhs) { return rhs / (*self) ; }
  int __cmp__(const BigFloat& rhs) { return cmp(*self, rhs); }
}; 

Expr sqrt(const Expr&);
bool isZero(const Expr&);
Expr fabs(const Expr&);
BigInt floor(const Expr&);
BigInt ceil(const Expr&);
long floorLg(const Expr&);
long ceilLg(const Expr&);
int sign(const Expr&);

int cmp(const Expr&, const Expr&);
int cmp(const Expr&, long);
int cmp(const Expr&, double);
int cmp(const Expr&, const std::string&);
int cmp(const Expr&, const BigInt&);
int cmp(const Expr&, const BigRat&);
int cmp(const Expr&, const BigFloat&);
int cmp(const Expr&, const Real&);
int cmp(long, const Expr&);
int cmp(double, const Expr&);
int cmp(const std::string&, const Expr&);
int cmp(const BigInt&, const Expr&);
int cmp(const BigRat&, const Expr&);
int cmp(const BigFloat&, const Expr&);
int cmp(const Real&, const Expr&);

